@@ -5,7 +5,8 @@ use std::slice::IterMut;
 
 use bevy::prelude::*;
 
-use crate::color::*;
+use crate::formatting::CharFormat;
+use crate::formatting::StringFormat;
 
 use sark_grids::Grid;
 
@@ -18,9 +19,9 @@ pub struct Tile {
     /// terminal's [UvMapping](super::renderer::uv_mapping::UvMapping)
     pub glyph: char,
     /// The forergound color for the tile.
-    pub fg_color: TileColor,
+    pub fg_color: Color,
     /// The background color for the tile.
-    pub bg_color: TileColor,
+    pub bg_color: Color,
 }
 
 /// A simple terminal for writing text in a readable grid.
@@ -49,8 +50,8 @@ impl Default for Tile {
     fn default() -> Self {
         Tile {
             glyph: ' ',
-            fg_color: WHITE,
-            bg_color: BLACK,
+            fg_color: Color::WHITE,
+            bg_color: Color::BLACK,
         }
     }
 }
@@ -58,6 +59,7 @@ impl Default for Tile {
 /// Border glyphs used in box drawing functions.
 ///
 /// Specifies the style of lines to use along the border of the box.
+#[derive(Clone, Copy)]
 pub struct BorderGlyphs {
     pub top: char,
     pub left: char,
@@ -70,7 +72,7 @@ pub struct BorderGlyphs {
 }
 
 /// Single line border glyphs. Can be used in box drawing functions.
-const SINGLE_LINE_GLYPHS: BorderGlyphs = BorderGlyphs {
+pub const SINGLE_LINE_GLYPHS: BorderGlyphs = BorderGlyphs {
     left: '│',
     right: '│',
     bottom: '─',
@@ -82,7 +84,7 @@ const SINGLE_LINE_GLYPHS: BorderGlyphs = BorderGlyphs {
 };
 
 /// Double line border glyphs. Can be used in box drawing functions.
-const DOUBLE_LINE_GLYPHS: BorderGlyphs = BorderGlyphs {
+pub const DOUBLE_LINE_GLYPHS: BorderGlyphs = BorderGlyphs {
     left: '║',
     right: '║',
     top: '═',
@@ -128,18 +130,6 @@ impl Terminal {
         self.tiles.pos_to_index(xy)
     }
 
-    /// Convert a 2D position to it's corresponding 1D index
-    /// in the terminal with the y axis flipped.
-    ///
-    /// In the terminal the y axis goes from top to bottom.
-    /// This should be used when passing in coordinates where
-    /// the y axis goes from bottom to top.
-    #[inline]
-    pub fn to_index_flipped(&self, xy: [i32; 2]) -> usize {
-        let (x, y) = self.y_flip(xy).into();
-        (y * self.width() as i32 + x) as usize
-    }
-
     /// Convert 1D index to it's 2D position given the dimensions
     /// of the terminal.
     ///
@@ -153,30 +143,6 @@ impl Terminal {
         IVec2::new(x, y)
     }
 
-    /// Convert 1D index to it's 2D position given the dimensions
-    /// of the terminal.
-    ///
-    /// In the terminal the y axis goes from top to bottom.
-    /// This should be used when passing in an index derived from
-    /// coordinates where the y axis goes from bottom to top.
-    #[inline]
-    pub fn to_xy_flipped(&self, i: usize) -> IVec2 {
-        let xy = self.to_xy(i);
-        self.y_flip(xy.into())
-    }
-
-    /// Flip the given position based on the height of the terminal.
-    ///
-    /// In the terminal the y axis goes from top to bottom.
-    /// This should be used when passing in coordinates where
-    /// the y axis goes from bottom to top.
-    #[inline]
-    pub fn y_flip(&self, pos: [i32; 2]) -> IVec2 {
-        let [x, y] = pos;
-        let y = self.height() as i32 - 1 - y;
-        IVec2::new(x, y)
-    }
-
     /// Insert a character.
     ///
     /// The existing foreground and background color of the tile will remain.
@@ -201,17 +167,10 @@ impl Terminal {
     }
 
     /// Insert a character with colors.
-    pub fn put_char_color(
-        &mut self,
-        xy: [i32; 2],
-        glyph: char,
-        fg_color: TileColor,
-        bg_color: TileColor,
-    ) {
-        let t = self.get_tile_mut(xy);
-        t.glyph = glyph;
-        t.fg_color = fg_color;
-        t.bg_color = bg_color;
+    pub fn put_char_formatted(&mut self, xy: [i32; 2], glyph: char, format: CharFormat) {
+        let xy = format.pivot.pivot_aligned_point(xy, self.size().into());
+        let t = self.get_tile_mut(xy.into());
+        *t = format.tile(glyph);
     }
 
     /// Insert a [Tile].
@@ -238,35 +197,28 @@ impl Terminal {
     ///
     /// The string will move to the next line if it reaches the edge
     /// and will truncate at the end of the terminal.
-    pub fn put_string_color(
-        &mut self,
-        xy: [i32; 2],
-        string: &str,
-        fg_color: TileColor,
-        bg_color: TileColor,
-    ) {
-        let i = self.to_index(xy);
+    pub fn put_string_formatted(&mut self, xy: [i32; 2], string: &str, format: StringFormat) {
+        let xy = format.get_string_position(xy, self.size.into(), string);
+        let i = self.to_index(xy.into());
         let tiles = self.tiles.slice_mut(i..).iter_mut().take(string.len());
         let chars = string.chars().take(tiles.len());
 
-        for (char, mut t) in chars.zip(tiles) {
-            t.glyph = char;
-            t.fg_color = fg_color;
-            t.bg_color = bg_color;
+        for (char, t) in chars.zip(tiles) {
+            *t = format.tile(char);
         }
     }
 
     /// Set the foreground color of a tile.
     ///
     /// The existing background color and glyph of the tile will remain.
-    pub fn put_fg_color(&mut self, xy: [i32; 2], col: TileColor) {
+    pub fn put_fg_color(&mut self, xy: [i32; 2], col: Color) {
         self.get_tile_mut(xy).fg_color = col;
     }
 
     /// Set the background color of a tile.
     ///
     /// The existing foreground color and glyph of the tile will remain.
-    pub fn put_bg_color(&mut self, xy: [i32; 2], col: TileColor) {
+    pub fn put_bg_color(&mut self, xy: [i32; 2], col: Color) {
         self.get_tile_mut(xy).bg_color = col;
     }
 
@@ -318,8 +270,8 @@ impl Terminal {
         let height = height as usize;
         let left = x as usize;
         let right = x as usize + width - 1;
-        let top = y as usize;
-        let bottom = y as usize + height - 1;
+        let bottom = y as usize;
+        let top = y as usize + height - 1;
 
         for t in self.row_iter_mut(top).skip(left).take(width) {
             t.glyph = border_glyphs.top;
@@ -327,10 +279,10 @@ impl Terminal {
         for t in self.row_iter_mut(bottom).skip(left).take(width) {
             t.glyph = border_glyphs.bottom;
         }
-        for t in self.column_iter_mut(left).skip(top).take(height) {
+        for t in self.column_iter_mut(left).skip(bottom).take(height) {
             t.glyph = border_glyphs.left;
         }
-        for t in self.column_iter_mut(right).skip(top).take(height) {
+        for t in self.column_iter_mut(right).skip(bottom).take(height) {
             t.glyph = border_glyphs.right;
         }
 
@@ -346,13 +298,12 @@ impl Terminal {
     }
 
     /// Draw a box with box with the specified colors and [BorderGlyphs].
-    pub fn draw_box_color(
+    pub fn draw_box_formatted(
         &mut self,
         xy: [i32; 2],
         size: [u32; 2],
-        fg_color: TileColor,
-        bg_color: TileColor,
         border_glyphs: BorderGlyphs,
+        format: CharFormat,
     ) {
         let [x, y] = xy;
         let [width, height] = size;
@@ -362,28 +313,20 @@ impl Terminal {
         let y = y as usize;
         let left = x;
         let right = x + width - 1;
-        let top = y;
-        let bottom = y + height - 1;
+        let bottom = y;
+        let top = y + height - 1;
 
         for t in self.row_iter_mut(top).skip(left).take(width) {
-            t.glyph = border_glyphs.top;
-            t.fg_color = fg_color;
-            t.bg_color = bg_color;
+            *t = format.tile(border_glyphs.top);
         }
         for t in self.row_iter_mut(bottom).skip(left).take(width) {
-            t.glyph = border_glyphs.bottom;
-            t.fg_color = fg_color;
-            t.bg_color = bg_color;
+            *t = format.tile(border_glyphs.bottom);
         }
-        for t in self.column_iter_mut(left).skip(top).take(height) {
-            t.glyph = border_glyphs.left;
-            t.fg_color = fg_color;
-            t.bg_color = bg_color;
+        for t in self.column_iter_mut(left).skip(bottom).take(height) {
+            *t = format.tile(border_glyphs.left);
         }
-        for t in self.column_iter_mut(right).skip(top).take(height) {
-            t.glyph = border_glyphs.right;
-            t.fg_color = fg_color;
-            t.bg_color = bg_color;
+        for t in self.column_iter_mut(right).skip(bottom).take(height) {
+            *t = format.tile(border_glyphs.right);
         }
 
         let left = left as i32;
@@ -391,35 +334,19 @@ impl Terminal {
         let top = top as i32;
         let bottom = bottom as i32;
 
-        self.put_char_color(
-            [left, bottom],
-            border_glyphs.bottom_left,
-            fg_color,
-            bg_color,
-        );
-        self.put_char_color([left, top], border_glyphs.top_left, fg_color, bg_color);
-        self.put_char_color([right, top], border_glyphs.top_right, fg_color, bg_color);
-        self.put_char_color(
-            [right, bottom],
-            border_glyphs.bottom_right,
-            fg_color,
-            bg_color,
-        );
+        self.put_char_formatted([left, bottom], border_glyphs.bottom_left, format);
+        self.put_char_formatted([left, top], border_glyphs.top_left, format);
+        self.put_char_formatted([right, top], border_glyphs.top_right, format);
+        self.put_char_formatted([right, bottom], border_glyphs.bottom_right, format);
     }
 
     /// Draw a box with a single-line border.
     pub fn draw_box_single(&mut self, xy: [i32; 2], size: [u32; 2]) {
-        self.draw_box(xy, size, SINGLE_LINE_GLYPHS);
+        self.draw_box_formatted(xy, size, SINGLE_LINE_GLYPHS, CharFormat::default());
     }
     /// Draw a box with a colored single-line border.
-    pub fn draw_box_single_color(
-        &mut self,
-        xy: [i32; 2],
-        size: [u32; 2],
-        fg_color: TileColor,
-        bg_color: TileColor,
-    ) {
-        self.draw_box_color(xy, size, fg_color, bg_color, SINGLE_LINE_GLYPHS);
+    pub fn draw_box_single_formatted(&mut self, xy: [i32; 2], size: [u32; 2], format: CharFormat) {
+        self.draw_box_formatted(xy, size, SINGLE_LINE_GLYPHS, format);
     }
 
     /// Draw a box with a double-line border.
@@ -427,14 +354,12 @@ impl Terminal {
         self.draw_box(xy, size, DOUBLE_LINE_GLYPHS);
     }
     /// Draw a box with a colored double-line border.
-    pub fn draw_box_double_color(
-        &mut self,
-        xy: [i32; 2],
-        size: [u32; 2],
-        fg_color: TileColor,
-        bg_color: TileColor,
-    ) {
-        self.draw_box_color(xy, size, fg_color, bg_color, DOUBLE_LINE_GLYPHS);
+    pub fn draw_box_double_formatted(&mut self, xy: [i32; 2], size: [u32; 2], format: CharFormat) {
+        self.draw_box_formatted(xy, size, DOUBLE_LINE_GLYPHS, format);
+    }
+
+    pub fn draw_border(&mut self, border_glyphs: BorderGlyphs) {
+        self.draw_box([0, 0], self.size().into(), border_glyphs);
     }
 
     /// Draw a single-line border around the edge of the whole terminal.
@@ -443,8 +368,8 @@ impl Terminal {
     }
 
     /// Draw a colored single-line border around the edge of the whole terminal.
-    pub fn draw_border_single_color(&mut self, fg_color: TileColor, bg_color: TileColor) {
-        self.draw_box_single_color([0, 0], self.size.into(), fg_color, bg_color);
+    pub fn draw_border_single_formatted(&mut self, format: CharFormat) {
+        self.draw_box_single_formatted([0, 0], self.size.into(), format);
     }
 
     /// Draw a double-line border around the edge of the whole terminal.
@@ -452,8 +377,39 @@ impl Terminal {
         self.draw_box_double([0, 0], self.size.into());
     }
     /// Draw a colored double-line border around the edge of the whole terminal.
-    pub fn draw_border_double_color(&mut self, fg_color: TileColor, bg_color: TileColor) {
-        self.draw_box_double_color([0, 0], self.size.into(), fg_color, bg_color);
+    pub fn draw_border_double_formatted(&mut self, format: CharFormat) {
+        self.draw_box_double_formatted([0, 0], self.size.into(), format);
+    }
+
+    pub fn draw_horizontal_bar(&mut self, xy: [i32; 2], width: i32, value: i32, max: i32) {
+        self.draw_horizontal_bar_color(xy, width, value, max, Color::WHITE, Color::GRAY);
+    }
+
+    pub fn draw_horizontal_bar_color(
+        &mut self,
+        xy: [i32; 2],
+        width: i32,
+        value: i32,
+        max: i32,
+        filled_color: Color,
+        empty_color: Color,
+    ) {
+        let [x, y] = xy;
+        let normalized = match max {
+            0 => 0.0,
+            _ => value as f32 / max as f32,
+        };
+
+        let v = f32::ceil(normalized * width as f32) as i32;
+
+        let filled_format = CharFormat::default().with_fg_color(filled_color);
+        let empty_format = CharFormat::default().with_fg_color(empty_color);
+        for i in 0..v {
+            self.put_char_formatted([x + i, y], '▓', filled_format);
+        }
+        for i in v..width {
+            self.put_char_formatted([x + i, y], '░', empty_format);
+        }
     }
 
     /// Clear the terminal tiles to default - empty tiles with
@@ -486,7 +442,7 @@ impl Terminal {
     }
 
     /// A mutable iterator over an entire row of tiles in the terminal.
-    pub fn row_iter_mut(&mut self, y: usize) -> IterMut<Tile> {
+    pub fn row_iter_mut(&mut self, y: usize) -> impl Iterator<Item = &mut Tile> {
         self.tiles.row_iter_mut(y)
     }
 
@@ -499,6 +455,26 @@ impl Terminal {
     pub fn column_iter_mut(&mut self, x: usize) -> StepBy<IterMut<Tile>> {
         self.tiles.column_iter_mut(x)
     }
+
+    /// The index of the bottom row of the terminal (0).
+    pub fn bottom_index(&self) -> usize {
+        0
+    }
+
+    /// The index of the top row of the terminal.
+    pub fn top_index(&self) -> usize {
+        self.height() as usize - 1
+    }
+
+    /// The index of the left-most column of the terminal (0).
+    pub fn left_index(&self) -> usize {
+        0
+    }
+
+    /// THe index of the right-most column of the terminal.
+    pub fn right_index(&self) -> usize {
+        self.width() as usize - 1
+    }
 }
 
 #[cfg(test)]
@@ -564,9 +540,9 @@ mod tests {
         let mut term = Terminal::with_size([10, 10]);
         term.draw_box_single([0, 0], [5, 5]);
 
-        assert_eq!(term.get_char([0, 0]), SINGLE_LINE_GLYPHS.top_left);
-        assert_eq!(term.get_char([0, 4]), SINGLE_LINE_GLYPHS.bottom_left);
-        assert_eq!(term.get_char([4, 0]), SINGLE_LINE_GLYPHS.top_right);
-        assert_eq!(term.get_char([4, 4]), SINGLE_LINE_GLYPHS.bottom_right);
+        assert_eq!(term.get_char([0, 4]), SINGLE_LINE_GLYPHS.top_left);
+        assert_eq!(term.get_char([0, 0]), SINGLE_LINE_GLYPHS.bottom_left);
+        assert_eq!(term.get_char([4, 4]), SINGLE_LINE_GLYPHS.top_right);
+        assert_eq!(term.get_char([4, 0]), SINGLE_LINE_GLYPHS.bottom_right);
     }
 }
